use std::str::FromStr;

use crate::{parse::Header, Error, Result};

#[derive(Clone, Copy, Debug)]
pub enum Compression {
    None,
    Bz2,
    Lz4,
}

impl FromStr for Compression {
    type Err = Error;

    fn from_str(s: &str) -> Result<Self, Self::Err> {
        Ok(match s {
            "none" => Compression::None,
            "bz2" => Compression::Bz2,
            "lz4" => Compression::Lz4,
            _ => return Err(Error::UnsupportedCompression(s.to_string())),
        })
    }
}

#[derive(Debug)]
pub struct ChunkHeader {
    pub compression: Compression,
    pub uncompressed_size: u32,
}

impl ChunkHeader {
    pub fn from_header(header: Header) -> Result<Self> {
        Ok(Self {
            compression: header.read_string(b"compression")?.parse()?,
            uncompressed_size: header.read_u32(b"size")?,
        })
    }
}
