use nom::{
    combinator::all_consuming,
    number::complete::{le_u32, le_u64, le_u8},
    Parser,
};
use num_enum::TryFromPrimitive;

use super::Input;
use crate::{Error, Result};

#[derive(Clone, Copy, Debug, TryFromPrimitive, PartialEq)]
#[repr(u8)]
pub enum Op {
    MsgData = 0x02,
    FileHeader = 0x03,
    IndexData = 0x04,
    Chunk = 0x05,
    ChunkInfo = 0x06,
    Connection = 0x07,
}

impl Op {
    pub fn parse(input: Input) -> Result<Self> {
        let (_, op) = all_consuming(le_u8).parse(input)?;
        Op::try_from_primitive(op).map_err(Into::into)
    }
}

pub fn parse_u32(input: Input) -> Result<u32> {
    let (_, x) = all_consuming(le_u32).parse(input)?;
    Ok(x)
}

pub fn parse_u64(input: Input) -> Result<u64> {
    let (_, x) = all_consuming(le_u64).parse(input)?;
    Ok(x)
}

pub fn parse_string(input: Input) -> Result<String> {
    String::from_utf8(input.to_owned()).map_err(|_| Error::other("invalid utf8"))
}
