use core::fmt;

use nom::{
    bytes::streaming::{is_a, tag},
    error::context,
    sequence::tuple,
    Parser,
};

use super::{IResult, Input};

#[derive(Clone, Debug, PartialEq)]
pub struct Version {
    pub major: u16,
    pub minor: u16,
}

impl fmt::Display for Version {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        write!(f, "{}.{}", self.major, self.minor)
    }
}

fn decimal_u16(input: Input) -> IResult<u16> {
    is_a("0123456789")
        .map(|a| String::from_utf8_lossy(a).parse::<u16>().unwrap())
        .parse(input)
}

impl Version {
    pub fn parse(input: Input) -> IResult<Self> {
        context(
            "Version Header",
            tuple((
                tag(b"#ROSBAG V"),
                decimal_u16,
                tag(b"."),
                decimal_u16,
                tag(b"\n"),
            ))
            .map(|(_, major, _, minor, _)| Version { major, minor }),
        )
        .parse(input)
    }
}
