use std::{fs::File, io};

use memmap::Mmap;

use super::BagReader;
use crate::{parse, Error, Result};

pub struct MmapReader {
    mmap: Mmap,
    pos: usize,
}

impl MmapReader {
    pub fn new(file: File) -> Result<Self> {
        // Safety: ¯\_(ツ)_/¯
        let mmap = unsafe { Mmap::map(&file) }?;
        Ok(Self { mmap, pos: 0 })
    }
}

impl BagReader for MmapReader {
    fn read_parser<'a, O: 'a, P>(&'a mut self, mut parser: P) -> Result<O>
    where
        P: nom::Parser<&'a [u8], O, parse::Error<&'a [u8]>>,
    {
        let input = self.mmap.get(self.pos..).ok_or(Error::Eof)?;
        match parser.parse(input) {
            Ok((rest, output)) => {
                self.pos += input.len() - rest.len();
                Ok(output)
            }
            Err(nom::Err::Incomplete(_)) => Err(Error::Eof),
            Err(nom::Err::Error(e) | nom::Err::Failure(e)) => Err(e.into()),
        }
    }

    fn seek(&mut self, pos: io::SeekFrom) -> Result<()> {
        match pos {
            io::SeekFrom::Start(pos) => {
                self.pos = pos as usize;
            }
            io::SeekFrom::End(pos) => {
                self.pos = (self.mmap.len() as i64 + pos) as usize;
            }
            io::SeekFrom::Current(pos) => {
                self.pos = ((self.pos as i64) + pos) as usize;
            }
        }
        Ok(())
    }
}
